//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/PolarizationAnalysisEditor.cpp
//! @brief     Implements class PolarizationAnalysisEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/PolarizationAnalysisEditor.h"
#include "GUI/Model/Item/DetectorItems.h"
#include "GUI/Model/Item/InstrumentItems.h"
#include "GUI/Model/Item/VectorItem.h"
#include "GUI/View/Instrument/VectorEditor.h"

#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGridLayout>
#include <QGroupBox>

PolarizationAnalysisEditor::PolarizationAnalysisEditor(QWidget* parent, InstrumentItem* instrument)
    : QWidget(parent)
    , m_instrument(instrument)
{
    ASSERT(instrument);

    auto* layout = new QVBoxLayout(this);

    m_checkbox = new QCheckBox("add polarizer and analyzer to instrument");
    connect(m_checkbox, &QCheckBox::toggled, this, &PolarizationAnalysisEditor::onCheckBoxToggled);
    layout->addWidget(m_checkbox);

    m_polarizerAnalyzerWidget = new QWidget(this);
    auto* grid = new QGridLayout(m_polarizerAnalyzerWidget);
    grid->setContentsMargins(0, 8, 0, 0);

    m_polarizerEditor = new VectorEditor("Polarizer (Bloch vector)",
                                         "Polarization of the beam, given as the Bloch vector (%1)",
                                         m_polarizerAnalyzerWidget);
    grid->addWidget(m_polarizerEditor, 0, 0);

    m_analyzerEditor =
        new VectorEditor("Analyzer orientation", "Direction of the polarization analysis (%1)",
                         m_polarizerAnalyzerWidget);
    grid->addWidget(m_analyzerEditor, 0, 1);

    auto* propertiesEditor = new QGroupBox("Analyzer properties", m_polarizerAnalyzerWidget);
    {
        auto* form = new QFormLayout(propertiesEditor);

        auto* m_efficiencySpinbox = new QDoubleSpinBox(propertiesEditor);
        m_efficiencySpinbox->setToolTip("Efficiency of the polarization analysis");
        m_efficiencySpinbox->setDecimals(4);
        m_efficiencySpinbox->setRange(std::numeric_limits<double>::lowest(),
                                      std::numeric_limits<double>::max());
        form->addRow("Efficiency:", m_efficiencySpinbox);

        auto* m_transmissionSpinbox = new QDoubleSpinBox(propertiesEditor);
        m_transmissionSpinbox->setToolTip("Total transmission of the polarization analysis");
        m_transmissionSpinbox->setDecimals(4);
        m_transmissionSpinbox->setRange(0, std::numeric_limits<double>::max());
        form->addRow("Transmission:", m_transmissionSpinbox);

        connect(m_efficiencySpinbox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
                &PolarizationAnalysisEditor::onEfficiencyValueChanged);
        connect(m_transmissionSpinbox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
                &PolarizationAnalysisEditor::onTransmissionValueChanged);
        m_efficiencySpinbox->setValue(instrument->analyzerEfficiency());
        m_transmissionSpinbox->setValue(instrument->analyzerTotalTransmission());
    }


    grid->addWidget(propertiesEditor, 0, 2);
    layout->addWidget(m_polarizerAnalyzerWidget);

    onCheckBoxToggled();

    m_checkbox->setChecked(m_instrument->withPolarizerAnalyzer());
    m_polarizerEditor->setVector(m_instrument->polarizationVector());
    m_analyzerEditor->setVector(m_instrument->analyzerDirectionVector());

    connect(m_polarizerEditor, &VectorEditor::vectorChanged, this,
            &PolarizationAnalysisEditor::dataChanged);
    connect(m_analyzerEditor, &VectorEditor::vectorChanged, this,
            &PolarizationAnalysisEditor::dataChanged);
}

void PolarizationAnalysisEditor::onCheckBoxToggled()
{
    const bool enabled = m_checkbox->isChecked();
    m_polarizerAnalyzerWidget->setEnabled(enabled);
    if (enabled != m_instrument->withPolarizerAnalyzer()) {
        m_instrument->setWithPolarizerAnalyzer(enabled);
        emit dataChanged();
    }
}

void PolarizationAnalysisEditor::onEfficiencyValueChanged(double efficiency)
{
    if (m_instrument->analyzerEfficiency() != efficiency) {
        m_instrument->setAnalyzerEfficiency(efficiency);
        emit dataChanged();
    }
}

void PolarizationAnalysisEditor::onTransmissionValueChanged(double transmission)
{
    if (m_instrument->analyzerTotalTransmission() != transmission) {
        m_instrument->setAnalyzerTotalTransmission(transmission);
        emit dataChanged();
    }
}
